#include "main.h"

typedef struct _Screen Screen;

struct _Screen
{
   struct {
      void (*start) (void *scr_data);
      void (*stop)  (void *scr_data);
      void (*go)    (void *scr_data);
      void (*ungo)  (void *scr_data);
   } func;
   
   Evas_Object *prev_focus;
   
   void *scr_data;
};

static Evas_List *screens = NULL;

void
screen_push(void (*start) (void *scr_data),
	    void (*stop)  (void *scr_data),
	    void (*go)    (void *scr_data),
	    void (*ungo)  (void *scr_data),
	    void *scr_data)
{
   Screen *s;
   
   if (screens)
     {
	s = screens->data;
	if (s->func.ungo) s->func.ungo(s->scr_data);
     }
   s = calloc(1, sizeof(Screen));
   s->func.start = start;
   s->func.stop = stop;
   s->func.go = go;
   s->func.ungo = ungo;
   s->scr_data = scr_data;
   if (!screens) s->prev_focus = evas_focus_get(evas);
   screens = evas_list_prepend(screens, s);
   if (s->func.start) s->func.start(s->scr_data);
}

void
screen_pop(void)
{
   Screen *s;
   
   if (!screens) return;
   s = screens->data;
   screens = evas_list_remove(screens, s);
   if (s->func.stop) s->func.stop(s->scr_data);
   if (s->prev_focus) evas_object_focus_set(s->prev_focus, 1);
   free(s);
   screen_go();
}

void
screen_go(void)
{
   Screen *s;
   
   if (!screens) return;
   s = screens->data;
   if (s->func.go) s->func.go(s->scr_data);
}

void
screen_ungo(void)
{
   Screen *s;
   
   if (!screens) return;
   s = screens->data;
   if (s->func.go) s->func.ungo(s->scr_data);
}

