#include "main.h"

typedef struct _Sc Sc;

struct _Sc
{
   unsigned char deleted : 1;
   Evas_Object *o_sc;
};

static void _sc_new(void *sc);
static void _sc_free(void *sc);
static void _sc_go(void *sc);
static void _sc_ungo(void *sc);

static void _sc_cb_key_down(void *data, Evas *e, Evas_Object *obj, void *event_info);
static void _sc_cb_stop_done(void *data, Evas_Object *o, const char *emission, const char *source);

Sc *cur_sc = NULL;

void
template_start(void)
{
   Sc *sc;
   
   sc = calloc(1, sizeof(Sc));
   screen_push(_sc_new, _sc_free, _sc_go, _sc_ungo, sc);
   screen_go();
}

/* == -- == */

static void
_sc_new(void *sc)
{
   Evas_Object *o;
   double w, h;
   
   cur_sc = sc;

   evas_output_viewport_get(evas, NULL, NULL, &w, &h);
   
   o = edje_object_add(evas);
   edje_object_file_set(o, ED"default.edj", "screen/template");
   evas_object_move(o, 0, 0);
   evas_object_resize(o, w, h);
   evas_object_event_callback_add(o, EVAS_CALLBACK_KEY_DOWN, _sc_cb_key_down, sc);
   edje_object_signal_callback_add(o, "stop", "done", _sc_cb_stop_done, sc);
   cur_sc->o_sc = o;
}

static void
_sc_free(void *sc)
{
   if (!cur_sc) return;
   cur_sc->deleted = 1;
   edje_object_signal_emit(cur_sc->o_sc, "stop", "");
}

static void
_sc_go(void *sc)
{
   if (!cur_sc) return;
   evas_object_focus_set(cur_sc->o_sc, 1);
   evas_object_show(cur_sc->o_sc);
   edje_object_signal_emit(cur_sc->o_sc, "start", "");
}

static void
_sc_ungo(void *sc)
{
   if (!cur_sc) return;
   edje_object_signal_emit(cur_sc->o_sc, "stop", "");
}

/* NB: on ipaq
 *
 * "Escape"
 *
 * "F1"   "F2"    "F3"   "F4"
 *            "Up"
 *   "Left" "Return" "Right"
 *           "Down"
 * 
 */
static void
_sc_cb_key_down(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
   Evas_Event_Key_Down *ev;
   
   if (!cur_sc) return;
   ev = event_info;
   if (!strcmp(ev->keyname, "Up"))
     {
     }
   else if (!strcmp(ev->keyname, "Down"))
     {
     }
   else if ((!strcmp(ev->keyname, "Left")) ||
	    (!strcmp(ev->keyname, "Escape")))
     {
	screen_pop();
     }
   else if ((!strcmp(ev->keyname, "Right")) ||
	    (!strcmp(ev->keyname, "Return")))
     {
     }
}

static void
_sc_cb_stop_done(void *data, Evas_Object *o, const char *emission, const char *source)
{
   Sc *sc;
   
   sc = data;
   
   evas_object_hide(cur_sc->o_sc);
   
   if (!sc->deleted) return;
   
   evas_object_del(sc->o_sc);
   free(sc);
}       
